'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const EFFECT_LIST = ['simple', 'random', 'camber', 'stagger', 'fence'];

var EffectDropdown = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:_vm.dropdownCls},[_c('span',{class:_vm.labelCls,style:(_vm.labelStyle),on:{"click":_vm.toggle}},[_vm._v(" "+_vm._s(_vm.label)+" "),_c('i',{staticClass:"effect-dropdown__label-caret effect-dropdown-iconfont"})]),_vm._v(" "),_c('ul',{staticClass:"effect-dropdown__content"},[_vm._t("default")],2)])},staticRenderFns: [],
  name: 'EffectDropdown',

  provide () {
    return {
      $$root: this,
    }
  },

  props: {
    effect: {
      default: 'simple',
      validator (value) {
        return !value || (typeof value === 'string' && EFFECT_LIST.indexOf(value) > -1)
      },
    },
    label: String,
    gutter: {
      type: Number,
      default: 5,
    },
    activeColor: {
      type: String,
      default: '#fc756f',
    },
    raiseLabel: Boolean,
    autoHide: {
      type: Boolean,
      default: true,
    },
  },

  data () {
    return {
      items: [],
      active: false,
    }
  },
  computed: {
    dropdownCls () {
      return {
        'effect-dropdown': true,
        'effect-dropdown--active': this.active,
      }
    },
    labelStyle () {
      return {
        color: this.active ? this.activeColor : null,
      }
    },
    labelCls () {
      return {
        'effect-dropdown__label': true,
        'effect-dropdown__label--raise': this.raiseLabel,
      }
    },
  },

  watch: {
    active (value) {
      if (this.autoHide) {
        if (value) {
          document.body.addEventListener('click', this.handleBodyClick);
        } else {
          document.body.removeEventListener('click', this.handleBodyClick);
        }
      }
    },
  },
  destroyed () {
    if (this.autoHide) {
      document.body.removeEventListener('click', this.handleBodyClick);
    }
  },

  methods: {
    addItem (vm, cb) {
      this.items.push(vm);

      typeof cb === 'function' && cb(this.items.length - 1);
    },
    toggle () {
      this.active = !this.active;
    },

    handleBodyClick (event) {
      const notOutside = this.$el.contains(event.target);

      if (!notOutside) {
        this.active = false;
      }
    },

    hide () {
      this.toggle();
    },
  },
}

const getTransformValue = (key, value) => `${key}(${value})`;

const transform = (values = {}) => {
  const result = [];

  Object.keys(values).forEach((key) => {
    const value = Array.isArray(values[key]) ? values[key].join(', ') : values[key];

    result.push(getTransformValue(key, value));
  });

  return result.join(' ')
};

const assign = (...sources) => Object.assign({}, ...sources);

const BASE = {
  zIndex: 1500,
  offset: 3,
  itemHeight: 60,
};

var EffectDropdownItem = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.index !== null)?_c('li',{class:_vm.classname,style:(_vm.itemStyle),on:{"click":_vm.handleClick,"mouseenter":_vm.handleMouseenter,"mouseleave":_vm.handleMouseleave}},[_c('span',{staticClass:"effect-dropdown-item__label",style:(_vm.labelStyle)},[_vm._t("default",[_vm._v(_vm._s(_vm.label))])],2)]):_vm._e()},staticRenderFns: [],
  name: 'EffectDropdownItem',

  inject: ['$$root'],

  props: {
    label: String,
    disabled: Boolean,
  },

  data () {
    return {
      index: null,
      hover: false,
      randomLeft: '',
    }
  },
  computed: {
    active () {
      return this.$$root.active
    },
    classname () {
      return {
        'effect-dropdown-item': true,
        [`effect-dropdown-item--${this.$$root.effect}`]: true,
        'effect-dropdown-item--disabled': this.disabled,
      }
    },
    labelStyle () {
      return {
        color: this.hover ? '#fff' : null,
        backgroundColor: this.hover ? this.$$root.activeColor : null,
      }
    },

    transform () {
      const { effect } = this.$$root;
      const scale = 1 - (this.index / 100);

      const commonOptions = {
        default: { scale3d: [scale, scale, 1] },
        active: { scale3d: [1, 1, 1] },
      };
      const optionsMap = {
        simple: assign(commonOptions),
        random: {
          default: assign(commonOptions.default, { rotate: 0 }),
          active: assign(commonOptions.active, { rotate: `${((Math.random() * 10) - 5).toFixed(2)}deg` }),
        },
        camber: {
          default: assign(commonOptions.default, { rotate: 0 }),
          active: assign(commonOptions.active, { rotate: `${this.index * 5}deg` }),
        },
        stagger: {
          default: {
            translate3d: [this.index % 2 ? '-40%' : '40%', 0, 0],
          },
          active: {
            translate3d: [0, 0, 0],
          },
        },
        fence: {
          default: {
            translate3d: [this.index % 2 ? '-50%' : '50%', 0, 0],
          },
          active: {
            translate3d: [0, 0, 0],
          },
        },
      };

      const defaultTransform = transform(optionsMap[effect].default);

      const activeTransfrom = transform(optionsMap[effect].active);

      return this.active ? activeTransfrom : defaultTransform
    },

    itemStyle () {
      const { effect, items } = this.$$root;
      const top = this.active ?
        `${(this.index + 1) * (BASE.itemHeight + this.$$root.gutter)}px` :
        `${BASE.offset * this.index}px`;

      const commonStyle = {
        top,
        transform: this.transform,
        zIndex: BASE.zIndex - (this.index + 1),
      };

      const styleMap = {
        simple: {},
        random: {
          left: this.randomLeft,
          'transition-delay': this.active ?
            `${(items.length - this.index - 1) * 100}ms` :
            `${this.index * 100}ms`,
        },
        camber: {
          top: this.active ?
            `${(BASE.itemHeight + this.$$root.gutter)}px` :
            commonStyle.top,
          'transition-delay': this.active ?
            `${(items.length - this.index - 1) * 50}ms` :
            `${this.index * 50}ms`,
        },
        stagger: {
          top: `${(this.index + 1) * (BASE.itemHeight + this.$$root.gutter)}px`,
          opacity: Number(this.active),
        },
        fence: {
          top: `${(this.index + 1) * (BASE.itemHeight + this.$$root.gutter)}px`,
          opacity: Number(this.active),
          'transition-delay': this.active ?
            `${this.index * 100}ms` :
            `${(items.length - this.index - 1) * 100}ms`,
        },
      };

      return assign(commonStyle, styleMap[effect])
    },
  },
  watch: {
    active (value) {
      this.hover = false;

      if (this.$$root.effect === 'random') {
        this.randomLeft = value ? `${Math.floor((Math.random() * 10) - 5)}px` : 0;
      }
    },
  },

  created () {
    this.$$root.addItem(this, (index) => {
      this.index = index;
    });
  },

  methods: {
    handleMouseenter (event) {
      if (this.disabled) return
      if (!this.active) return

      this.hover = true;

      this.$emit('mouseenter', this, event);
    },
    handleMouseleave (event) {
      if (this.disabled) return

      this.hover = false;

      this.$emit('mouseleave', this, event);
    },

    handleClick (event) {
      if (this.disabled) return

      this.$emit('click', this, event);

      this.$$root.autoHide && this.$$root.hide();
    },
  },
}

// export { EffectDropdown }

var index = {
  install (Vue) {
    Vue.component(EffectDropdown.name, EffectDropdown);
    Vue.component(EffectDropdownItem.name, EffectDropdownItem);
  },
}

exports['default'] = index;
